/*
 * Copyright (C) 2004-2005 Andre Noll <maan@systemlinux.org>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "server.h"
#include <regex.h>
#include <pwd.h>

void *para_realloc(void *p, size_t size)
{
	/* 
	 * No need to check for NULL pointers: If p is NULL, the  call
	 * to realloc is equivalent to malloc(size)
	 */
	if (!(p = realloc(p, size))) {
		clog(EMERG, "%s: realloc failed, aborting", __func__);
		exit(EXIT_FAILURE);
	}
	return p;
}
void *para_malloc(size_t size)
{
	void *p = malloc(size);

	if (!p) {
		clog(EMERG, "%s: malloc failed, aborting", __func__);
		exit(EXIT_FAILURE);
	}
	return p;
}

char *para_strdup(const char *s)
{
	char *ret;

	if ((ret = strdup(s? s: "")))
		return ret;
	clog(EMERG, "%s: strdup failed, aborting", __func__);
	exit(EXIT_FAILURE);
}

/* taken from printf man page */
char *make_message(const char *fmt, ...) 
{
	/* Guess we need no more than 100 bytes. */
	int n, size = 100;
	char *p = para_malloc(size);
	va_list ap;
	while (1) {
		/* Try to print in the allocated space. */
		va_start(ap, fmt);
		n = vsnprintf(p, size, fmt, ap);
		va_end(ap);
		/* If that worked, return the string. */
		if (n > -1 && n < size)
			return p;
		/* Else try again with more space. */
		if (n > -1) /* glibc 2.1 */
			size = n + 1; /* precisely what is needed */
		else /* glibc 2.0 */
			size *= 2; /* twice the old size */
		p = para_realloc(p, size);
	}
}

char *para_strcat(char *a, const char *b)
{
	char *tmp;
	
	if (!a) 
		return para_strdup(b);
	if (!b)
		return a;
	tmp = make_message("%s%s", a, b);
	free(a);
	return tmp;
}

char *para_basename(char *name)
{
	char *p;

	if (!name || !*name)
		return NULL;
	p = strrchr(name, '/'); 
	if (!p)
		return para_strdup(name);
	p++;
	if (!*p)
		return NULL;
	return para_strdup(p);
}

/*
 * Simple search and replace routine
 */
char *s_a_r(const char *src, const char* regex, const char *replacement)
{
	regex_t preg;
	size_t  nmatch = 1;
	regmatch_t pmatch[1];
	int eflags = 0;
	char *dest = NULL;
	const char *bufptr = src;

//	clog(DEBUG, "%s: src: %s  regex: %s repl: %s\n", __func__, src,
//		regex, replacement);
	if (!regex || !replacement || !src)
		return para_strdup(src);
	regcomp(&preg, regex, 0);
	while (regexec(&preg,  bufptr, nmatch, pmatch, eflags) 
			!= REG_NOMATCH) {
		char *tmp, *arg, *o_bracket, *c_bracket;
	
		o_bracket = strchr(bufptr + pmatch[0].rm_so, '(');
		c_bracket = o_bracket? strchr(o_bracket, ')') : NULL;
		if (!c_bracket)
			goto out;
		tmp = para_strdup(bufptr); 
		tmp[pmatch[0].rm_so] = '\0';
		dest = para_strcat(dest, tmp); 
		free(tmp);

		arg = para_strdup(o_bracket + 1);
		arg[c_bracket - o_bracket - 1] = '\0';
		tmp = make_message(replacement, arg);
		free(arg);
		dest = para_strcat(dest, tmp);
		free(tmp);
		bufptr = c_bracket;
		bufptr++;
	}
	dest = para_strcat(dest, bufptr);
//	clog(DEBUG, "%s: returning %s\n", __func__, dest);
out:
	regfree(&preg);
	return dest;
}

/*
 * search src and replace to dest according to each entry in regex_list 
 */
char *s_a_r_list(struct regex_list *regex_list, char *src)
{
	struct regex_list *rp = regex_list; 
	char *ret = NULL, *tmp = para_strdup(src);
	
	while (rp->regex) {
		ret = s_a_r(tmp, rp->regex, rp->replacement);
		free(tmp);
		if (!ret)
			return src;
		tmp = ret;
		rp++;
	}
	//clog(DEBUG, "%s: returning %s\n", __func__, dest);
	return ret;
}

void chop(char* buf)
{
	int n = strlen(buf);
	if (!n)
		return;
	if (buf[n - 1] == '\n')
		buf[n - 1] = '\0';
}


int para_fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	size_t res = fread(ptr, size, nmemb, stream);
	if (res == nmemb)
		return size * nmemb;
	if (feof(stream))
		return 0;
	return -1;
}

/* 
 * This is by no means a secure way to create temporary files in a
 * hostile direcory like /tmp.  However, it is OK to use it for temp
 * files, fifos, sockets that are created in ~/.paraslash.
 *
 */
char *para_tmpname(void)
{
	struct timeval now;
	gettimeofday(&now, NULL);
	srand(now.tv_usec);
	return make_message("%08i", rand());
}

int para_mkstemp(char *template, mode_t mode)
{
	int tmp, fd = mkstemp(template);
	
	if (fd < 0)
		return fd;
	tmp = fchmod(fd, mode);
	if (tmp >= 0)
		return fd;
	close(fd);
	unlink(template);
	return -1;
}

char *concat_args(const int argc, char * const *argv)
{
	int i; char *buf = NULL;
	for (i = 0; i < argc; i++) {
		buf = para_strcat(buf, argv[i]);
		buf = para_strcat(buf, "\n");
	}
	return buf;
}

char *para_logname(void)
{
	struct passwd *pw = getpwuid(getuid());
	return para_strdup(pw? pw->pw_name : "unknown_user");
}

char *para_homedir(void)
{
	struct passwd *pw = getpwuid(getuid());
	return para_strdup(pw? pw->pw_dir : "/tmp");
}

/*
 * construct string containing uptime
 */
char *uptime_str(int i)
{
	return make_message("%i:%02i:%02i", i / 86400,
		(i / 3600) % 24, (i / 60) % 60);
}
